import numpy as np
from sklearn.utils import resample
import random
import matplotlib.pyplot as plt


# Set the mean and standard deviation of the Gaussian distribution
mean = [0,0]
cov = [[1,0],[0,1]]

# Generate a Gaussian vector with 100 elements


#ALGO 1 : SSPD
#w vect de taille N
# X de dimension d
#N nombre de repetitons dans la chaîne de Markov

def init(d,n):
    X0 = np.zeros((d,n))
    Y0 = np.zeros((d,n))
    for i in range(len(X0)):
        X0[i] = np.array([-1 + random.randint(-100,100)*0.0025,0 + random.randint(-100,100)*0.0025])
    for i in range(len(Y0)):
        Y0[i] = np.array([0.1,0.1])
    return X0,Y0

def draw_X(X):
    for i in range(len(X)):
        plt.scatter(X[i][0],X[i][1], color='red', marker='+')

def ESS(w):
    N = len(w)
    numerateur = 0
    denominateur = 0
    for n in range(1,N):
        numerateur += w[n]
        denominateur += w[n]**2
    return numerateur**2/denominateur

def V(X):
    return (1-X[0]**2)**2 + 2*X[1]**2
def V_2d(X,Y):
    return V([X,Y])

def grad_V(X):
    return np.array([-4*X[0]*(1-X[0]**2),4*X[1]])

def Hess_V(X):
    return np.array([[-4+12*X[0]**2,0],[0,4]])


def draw(X):
    # Figure Drawing :
    x = np.linspace(-2, 2, 1000)
    y = np.linspace(-2, 2, 1000)
    XX, YY = np.meshgrid(x, y)
    ZZ = V_2d(XX, YY)
    fig, ax = plt.subplots()
    ax.set_xlabel('x')
    ax.set_ylabel('y')
    m1 = (-1, 0)
    m2 = (1, 0)
    s1 = (0, 0)
    plt.plot(*m1, 'm*'), plt.text(*m1, "  Min 1")
    plt.plot(*m2, 'm*'), plt.text(*m2, "  Min 2")
    plt.plot(*s1, 'bo'), plt.text(*s1, "  Saddle 1")

    c = ax.contourf(XX, YY, ZZ)
    plt.colorbar(c)
    draw_X(X)


def SSPD(K,N,X0,Y0,rho_ess=0.9,BETA = 10,delta = 1e-3):
    X = np.zeros((N,K,2))
    Y = np.zeros((N,K,2))
    for i in range(len(X0)):
        X[i][0] = X0[i]
    for i in range(len(Y0)):
        Y[i][0] = Y0[i]
    W = np.zeros((N,K))
    for k in range(K-1):
        G = np.random.multivariate_normal(mean,cov,size=(N,K))
        for n in range(N):
            W[n][k] = np.sqrt(Y[n][k][0]**2 + Y[n][k][1]**2)
        if ESS(W[:,k]) < rho_ess*N:
            W_sum = 0
            for i in range(len(W[:,k])):
                W_sum+= W[i,k]
            W_norm = W[:,k]/W_sum
            I = [i for i in range(N)]
            J = [np.random.choice(I,p=W_norm) for i in range(N)]
            for n in range(N):
                X[n][k] = X[J[n]][k]
                Y[n][k] = Y[J[n]][k]/W[J[n]][k]
                pass
        for n in range(N):
            delta_x=0.0001
            delta_y=0.01
            X[n][k+1] = X[n][k] - delta_x * grad_V(X[n][k]) + np.sqrt(2*BETA**(-1)*delta)*G[n][k]
            Y[n][k+1] = Y[n][k] - delta_y * np.dot(Hess_V(X[n][k]), Y[n][k])
        if k%100 == 0:
            draw(X[:,k,:])
            plt.show()

X0, Y0 = init(50,2)
SSPD(1000,50,X0,Y0)