#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sat Dec  7 20:20:48 2019

@author: p.parpas
"""

import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
from matplotlib import cm
# from io import StringIO
import numpy as np
# from mpl_toolkits import mplot3d


plt.rcParams.update({
    "text.usetex": True,
    "font.family": "sans-serif",
    "font.sans-serif": ["Helvetica"]})
# for Palatino and other serif fonts use:
plt.rcParams.update({
    "text.usetex": True,
    "font.family": "serif",
    "font.serif": ["Palatino"],
})

def double_well(x,y):
    E=2*10**(-4)
    C=0.045
    mu=0.001
    f=E*(C*x**4-x**2)+mu*y**2
    return f


#%matplotlib qt
T=10000
plot_V=False
plotP=True


fig = plt.figure()
# plt.tight_layout()
fig.tight_layout()
# ax = fig.add_subplot(111)
fig.subplots_adjust(left=0.05, bottom=0.05, right=0.9, top=0.98)
if plotP is True:
    
    plt.rcParams["figure.figsize"] = [12,7]
    # plt.autoscale(tight=True)
    
    have_py=True
    
    if have_py is True:
      ax_l = fig.add_subplot(1,2,1, projection='3d')
      ax_r = fig.add_subplot(1,2,2, projection='3d')
    
    else:
     ax_l = fig.add_subplot(1,1,1, projection='3d')
   
    # T=200
    T=2
    for t in range(1,T):
        i=t*100000
        if t==T:
            i=i-1
            
        file="../pde/p_r"+str(i)+".csv"
#         file="../pde_fokker/p_r"+str(i)+".csv"

        try:
          # print(file)
          data = np.genfromtxt(file,dtype="float", delimiter=",")
          X=data[:,0]
          Y=data[:,1]
          P_x=data[:,2]
          if have_py is True:
            P_y=data[:,3]
    
    
  
    # ax.scatter(X, Y,P)  #
    # ax.plot_trisurf(X, Y,P, cmap=plt.cm.viridis, linewidth=0.2)
    # cmap=plt.cm.jet
    
          nx=int(np.sqrt(X.shape[0]))
          ny=int(np.sqrt(Y.shape[0]))
          P_gx=np.zeros((nx,ny))
          if have_py is True:
            P_gy=np.zeros((nx,ny))
        
          X_g=np.zeros((nx,ny))
          Y_g=np.zeros((nx,ny))
        
          X_g=np.linspace(X[0], X[-1], nx) 
          Y_g=np.linspace(Y[0], Y[-1], ny) 
        
          [X, Y] = np.meshgrid(X_g,Y_g)
        
          
          s_x=sum(P_x)
          s_y=0
          if have_py is True:
            s_y=sum(P_y)
          k=0
          for i in range(0,nx):
                for j in range(0,ny):
                    P_gx[i,j]=P_x[k]/s_x
                    if have_py is True:
                      P_gy[i,j]=P_y[k]
                    k=k+1
         
        # ax.contour(X_g, Y_g,P_g,1000,cmap='RdGy')  
          ax_l.clear()
          t_string="t="+"{:.5f}".format(t)
          t_string+=" Sx="+"{:.5f}".format(s_x)
          t_string+=" Sy="+"{:.5f}".format(s_y)
          # ax_l.title.set_text(t_string)
          c_x=np.max(P_gx)
          if c_x<1:
             c_s=1+int(np.log10(1/c_x))
             P_gx=P_gx*10**(c_s)
             ztitle = '$\phi_1(x_1,x_2)\\times 10^{-'+str(c_s)+'}$'
             # ax_l.text(0, 2, '$times 10^{-'+str(c_s)+'}$')
          else:
              ztitle = '$\phi_1(x_1,x_2)$'
          print(t,s_x,s_y)

          # if t>10:
          #     P_gx=P_gx*10**(4)
          #     ztitle = '$\psi_1(x_1,x_2)$'
          ax_l.plot_surface(X,Y, P_gx,rstride=8, cstride=8, alpha=0.3)
          cset = ax_l.contourf(X_g,Y_g, P_gx,20, zdir='z', offset=0, cmap=cm.coolwarm,zorder=0)
          cset = ax_l.contourf(X,Y, P_gx, 20,zdir='x', offset=-10, cmap=cm.coolwarm)
          cset = ax_l.contourf(X,Y, P_gx,20, zdir='y', offset=10, cmap=cm.coolwarm)
          # ax_l.set_major_formatter(FormatStrFormatter('% 1.1f'))
          # ax_l.scatter(0,0,0)
          # ax_l.scatter(0, 3.33, c='k', zorder=100)
          # ax_l.scatter(0,-3.33)
          ax_l.set_xlabel('$x_2$')
          ax_l.set_ylabel('$x_1$')
          ax_l.set_zlabel(ztitle)
          #
          # ax_l.set_zlabel('$\\rho(x_1,x_2)\\times 10^{-5} $')
          # zlim=ax_l.get_zlim()
          # plt.yticks([])
          # plt.xticks([])
          # ax_l.set_zticks([])

          if have_py is True:
            ax_r.clear()
            ztitle='$\psi_2(x_1,x_2)$'
            c_y = np.max(P_gy)
            if c_y < 1:
                c_s = 1 + int(np.log10(1 / c_y))
                P_gy = P_gy * 10 ** (c_s)
                ztitle = '$\psi_2(x_1,x_2)\\times 10^{-' + str(c_s) + '}$'
            else:
                ztitle = '$\psi_2(x_1,x_2)$'
            print(t, s_x, s_y)


            ax_r.plot_surface(X, Y, P_gy, rstride=8, cstride=8, alpha=0.3)
            cset = ax_r.contourf(X_g, Y_g, P_gy, 20, zdir='z', offset=0, cmap=cm.coolwarm, zorder=0)
            cset = ax_r.contourf(X, Y, P_gy, 20, zdir='x', offset=-10, cmap=cm.coolwarm)
            cset = ax_r.contourf(X, Y, P_gy, 20, zdir='y', offset=10, cmap=cm.coolwarm)
            ax_r.set_xlabel('$x_2$')
            ax_r.set_ylabel('$x_1$')
            ax_r.set_zlabel(ztitle)
            # ax_r.set_zlim(zlim[0],zlim[1])

          # plt.tight_layout()
          plt.show()
          # plt.pause(0.0001)
          plt.savefig(str("../pde/pde_vec_"+str(t)+".png"), dpi=600,pad_inches=0)

        except IOError:
          print("File not found=",file)
#/Applications/ffmpeg -i pde_vec_%d.png -vf scale="trunc(iw/2)*2:trunc(ih/2)*2" -c:v libx264 -profile:v high -pix_fmt yuv420p -g 30 -r 30 2d_pde_FP.mp4


#if plot_V is True:
#    ax_l = fig.add_subplot(1,1,1, projection='3d')
#    file="px0.csv"
#    data = np.genfromtxt(file,dtype="float", delimiter=",")
#    X=data[:,0]
#    Y=data[:,1]
#    nx=int(np.sqrt(X.shape[0]))
#    ny=int(np.sqrt(Y.shape[0]))
#    Z_g=np.zeros((nx,ny))
#    X_g=np.zeros((nx,ny))
#    Y_g=np.zeros((nx,ny))
#    
#    X_g=np.linspace(X[0], X[-1], nx) 
#    Y_g=np.linspace(Y[0], Y[-1], ny) 
#    
#    [X, Y] = np.meshgrid(X_g,Y_g)
#    
#   
#    for i in range(0,nx):
#          for j in range(0,ny):
#              Z_g[i,j]=double_well(X_g[i],Y_g[j])
#
#    ax_l.clear()
#    ax_l.plot_surface(X,Y, Z_g, rstride=8, cstride=8, alpha=0.3)
#    cset = ax_l.contourf(X_g,Y_g, Z_g, zdir='z', offset=0, cmap=cm.coolwarm)
#    cset = ax_l.contourf(X,Y, Z_g, zdir='x', offset=-10, cmap=cm.coolwarm)
#    cset = ax_l.contourf(X,Y, Z_g, zdir='y', offset=10, cmap=cm.coolwarm)
#    ax_l.set_xlabel('X')
#    ax_l.set_ylabel('Y')
#    ax_l.set_zlabel('P')
#    
#    
#    plt.title("2d well -- no cross")
#    plt.show()
#   
#    plt.savefig(str("2dwell_no_cross.png"), dpi=200)