import matplotlib
import numpy as np
from scipy import stats
matplotlib.use('Agg')
import matplotlib.pyplot as plt
import pickle
import importlib
import matplotlib.cm as cm
import jax.scipy.stats.norm as jaxnm

import jax.numpy as jnp
from datetime import datetime
from scipy.spatial import ConvexHull
from jax import vmap,jit
import dimmer_utils
import func_utils
import potentials_2d
import time
importlib.reload(potentials_2d)
importlib.reload(func_utils)
importlib.reload(dimmer_utils)

def plot_text(ax_t,prms,xs,ys):
    plot_params=make_parameter_string(prms)
    ax_t.text(xs,ys,plot_params,fontsize=9)
    plt.tight_layout()
    plt.savefig(prms.str_file+str(0)+str(".png"),dpi=100)


def make_parameter_string(prms):

  now = datetime.now()
  dt_string = now.strftime("%d/%m/%Y %H:%M:%S")
  plot_str = "time:"+dt_string+"\n"
  dicprms=vars(prms)
  listprms=list(dicprms)
  for i in listprms:
#      key_name=listprms[i]
      plot_str+=str(i)+'='
      plot_str+=str(dicprms[i])+'\n'
  return plot_str

# load parameters

plt.rcParams.update({
    "text.usetex": True,
    "font.family": "sans-serif",
    "font.sans-serif": ["Helvetica"]})
# for Palatino and other serif fonts use:
plt.rcParams.update({
    "text.usetex": True,
    "font.family": "serif",
    "font.serif": ["Palatino"],
})

file_name="../example_challenge_case/dgn3_prms.pickle"
# file_name="../example_1/sdw_prms.pickle"
# file_name="../example_5/dgn3_prms.pickle"
# file_name="../muller_brown_01/mb1_prms.pickle"
# file_name="../muller_brown_01p/mb1_prms.pickle"
# file_name="../muller_brown_02p/mb1_prms.pickle"
# file_name="../muller_brown_02/mb1_prms.pickle"


file = open(file_name, 'rb')
plot_index1=True
h = 0.01
levels=50
prms=pickle.load(file)
file.close()
prms,V,V_plot,x_s, y_s, l_s,X0,limits=potentials_2d.setup_problem(prms)
T=np.int32(prms.T_real/prms.h_x)
Vm,dV_s,ddV_s,dV,hvp_vec,ddV,index_func_lambda,index_func_lambda_map,gd_map=func_utils.get_func_evals(V,prms)

#create contour plot
plt.figure(figsize=(400, 400))
plt.rcParams.update({'font.size': 6})

plt.rc('xtick', labelsize=6)
plt.rc('ytick', labelsize=6)
fig, axs = plt.subplots(1,1)

plot_text([axs][0],prms,0.1,0.0)
plt.savefig(prms.str_file+str("_prms.png"), dpi=100)
# plt.rcParams.update({'axes.titlesize': 'small'})
num_plots=1
fig, axs = plt.subplots(1,num_plots)
fig.tight_layout()
if num_plots==1:
    axs=[axs]
x_min,x_max,y_min,y_max=limits[0], limits[1],limits[2], limits[3]
for i in range(0,num_plots):
    axs[i].set(xlim=(x_min, x_max), ylim=(y_min, y_max))
    axs[i].set_aspect('equal', 'box')




# positions = np.vstack([X.ravel(), Y.ravel()])

x = np.arange(x_min, x_max, h)
y = np.arange(y_min, y_max, h)
X, Y = np.meshgrid(x, y)
# compute regions where hessian is index-1
if plot_index1:
    index1_val = np.zeros(X.shape)
    for ii in range(0,X.shape[0]):
        for jj in range(0, X.shape[0]):
            X_eval = jnp.array([X[ii, jj],Y[ii, jj] ]).reshape(1,2)
            ddv_eval = jnp.array(ddV_s(X_eval))
            eig_val = jnp.linalg.eigvalsh(ddv_eval.reshape(2,2))
            if eig_val[0] < 0 and eig_val[1] > 0:
                index1_val[ii, jj] = 10
    axs[0].contourf(X, Y, index1_val, 1, cmap=cm.binary, zorder=5, alpha=0.2)
    # axs[0,0].plot(index1_points[simplex, 0], index1_points[simplex, 1], 'k-',lw=2, zorder=10)
# print("Done")
# h= 0.01
# x = np.arange(x_min, x_max,h)
# y= np.arange(y_min, y_max,h)
# X, Y = np.meshgrid(x, y)
Vgrid=V_plot(X,Y)

colors = cm.gist_rainbow(jnp.linspace(0, 1, prms.N_a))
axs[0].contour(X, Y, Vgrid, levels=levels, cmap=plt.cm.coolwarm,zorder=5, linestyles='-')
axs[0].scatter(X0[0],X0[1],c=colors,s=10,zorder=5)

axs[0].scatter(3.72369054,3.75280605,c=colors,s=10,zorder=5)
axs[0].scatter(5,5,c=colors,s=10,zorder=5)


plot_eig_vectors=False

axs[0].set_aspect('equal', 'box')
fname=prms.str_file + "_V_0.svg"
plt.savefig(fname, dpi=300)
# N_f=388
N_f=1
for fs in range(1,N_f):
    t1 = time.perf_counter()
    # load states
    # print(fs)
    file_x = open(prms.str_file + "x_stats_" + str(fs) + ".pickle", "rb")
    # print(file)
    [out_s,t,X_t,Y_t,wgts,run_dimmer,S_card]=pickle.load(file_x)

    file_x.close()
    plot_dimmer = run_dimmer is True and S_card>0
    if plot_dimmer:
        file_d = open(prms.str_file + "d_stats_" + str(fs) + ".pickle", "rb")
        [U_0,t, exit_code,dimmer_iters,x_index1,x_final] = pickle.load(file_d)
        file_d.close()

    axs[0].clear()
    axs[1].clear()

    # plot function contours
    axs[0].contour(X, Y, Vgrid, levels=levels, cmap=plt.cm.coolwarm, linestyles='-',zorder=0)
    # plot particles with colours coded according to weight
    axs[0].scatter(X_t[:,0,0], X_t[:,0,1],cmap=plt.cm.coolwarm, c=wgts.W, s=5,zorder=5)
    if plot_index1:
        axs[0].contourf(X, Y, index1_val, 1, cmap=cm.binary, zorder=2, alpha=0.2)

    # plot avg y
    avg_x = jnp.sum(X_t.reshape(prms.N_p,prms.d*prms.N_a)*wgts.W, axis=0)
    avg_y = jnp.sum(Y_t.reshape(prms.N_p, prms.d * prms.N_a) * wgts.W, axis=0)
    norm_avg_y=jnp.linalg.norm(avg_y)
    avg_y=avg_y/norm_avg_y
    scale_factor=0.2
    if plot_eig_vectors:
        axs[0].arrow(avg_x[0],  avg_x[1], scale_factor*avg_y[0],scale_factor*avg_y[1],color='k',head_width=0.1,zorder=15)
    axs[0].scatter(avg_x[0], avg_x[1], c='k', s=50,marker=".",zorder=10)

    # plot smallest eigenvector at current avg x
    ddV_avg=ddV_s(avg_x.reshape(1,2)).reshape(2,2)
    eigval_avg,eigvec_avg=jnp.linalg.eigh(ddV_avg)
    # if
    norm_eigvec=jnp.linalg.norm(eigvec_avg[0])
    eigvec_avg_1=eigvec_avg[0]/norm_eigvec
    theta_pos=jnp.arccos(jnp.dot(avg_y,eigvec_avg_1))
    theta_neg = jnp.arccos(-jnp.dot(avg_y, eigvec_avg_1))
    # theta_pos=jnp.linalg.norm(eigvec_avg[0]-avg_y)
    # theta_neg = jnp.linalg.norm(-eigvec_avg[0] - avg_y)
    if theta_neg<theta_pos:
        eigvec_avg_1=-eigvec_avg_1

    # if plot_eig_vectors:
    #     axs[0].arrow(avg_x[0],  avg_x[1], scale_factor * eigvec_avg_1[0], scale_factor * eigvec_avg_1[1], color='r', head_width=0.1, zorder=5)

    if plot_dimmer is True:
        # initial condition(s)
        if len(U_0.shape)>2:
            axs[0].scatter(U_0[:,0,0], U_0[:,0,1], c='k', s=20,marker="x",zorder=9,alpha=0.2)
        else:
            axs[0].scatter(U_0[0, 0], U_0[0, 1], c='k', s=20, marker="x", zorder=9,alpha=0.2)
        dimmer_colour='g'
        # 0:success,1:hit max iter,2:failed because out of index-1
        if exit_code==0:
            dimmer_colour = 'green'
        elif exit_code == 1:
            dimmer_colour = 'grey'
        elif exit_code == 2:
            dimmer_colour = 'red'

        if exit_code == 0 and len(x_index1.shape) > 2:
            if exit_code == 0:
                axs[0].scatter(x_index1[:,0, 0], x_index1[:,0, 1], c=dimmer_colour,s=50,marker="X",zorder=20)
        elif exit_code==0:
            axs[0].scatter(x_index1[0, 0], x_index1[0, 1], c=dimmer_colour, s=50, marker="X", zorder=20)

        if exit_code!=0  and len(x_final.shape) > 2:
            axs[0].scatter(x_final[:, 0, 0], x_final[:, 0, 1], c=dimmer_colour, s=50, marker="X", zorder=20)
        elif exit_code != 0:
            axs[0].scatter(x_final[0, 0], x_final[0, 1], c=dimmer_colour, s=50, marker="X", zorder=20)

    axs[0].set_aspect('equal', 'box')

    fig.suptitle(out_s, fontsize=7)
    fname = prms.str_file + "_V_"+str(fs)+".png"

    kappa1 = np.zeros(Vgrid.shape)
    kappa2 = np.zeros(Vgrid.shape)

    sigma=0.01
    X,Y=jnp.array(X),jnp.array(Y)
    prtcl_index = jnp.arange(0, prms.N_p)
    x1_t, x2_t = jnp.array(X_t[:, 0, 0]), jnp.array(X_t[:, 0, 1])
    y1_t, y2_t = jnp.array(Y_t[:, 0, 0]), jnp.array(Y_t[:, 0, 1])

    kappa_1i = lambda k: y1_t[k]*jnp.exp(-( (X-x1_t[k])**2+(Y-x2_t[k])**2)/sigma)
    kappa_2i = lambda k: y2_t[k]*jnp.exp(-((X - x1_t[k]) ** 2 + (Y - x2_t[k]) ** 2) / sigma)
        # Y_t[k, 0, 0]*jnp.exp(-( (X-x1_t[k])**2+(Y-x2_t[k])**2)/sigma)
    kappa_1i=jit(kappa_1i)
    kappa_2i = jit(kappa_2i)
    kappa1 = jnp.sum(vmap(kappa_1i)(prtcl_index),axis=0)
    kappa2 = jnp.sum(vmap(kappa_2i)(prtcl_index), axis=0)
    ratio=jnp.linalg.norm(kappa1,ord='fro')/jnp.linalg.norm(kappa1+kappa2,ord='fro')
    # ratio = str('{:>2.2f}'.format(ratio))
    axs[1].text(-1,1,"$$\\frac{{\|\kappa_1\|}}{{\|\kappa_1+\kappa_2\|}}={:>2.2f}$$".format(ratio))
    # axs[1].text(-1, 1, "$$\kappa_1={:>2.2f}$$".format(1.0))
    pos1 = axs[1].contour(X, Y, kappa1 + kappa2, 20, cmap=cm.coolwarm)

    for i in range(0, 2):
        axs[i].set(xlim=(x_min, x_max), ylim=(y_min, y_max))
        axs[i].set_aspect('equal', 'box')
    plt.savefig(fname, dpi=300)
    t2 = time.perf_counter()
    print(fs, t2 - t1)
    # #add plot for kappa(x,t)\approx \sum w_i delta(x-x_i)/N
    # x1_t, x2_t = X_t[:, 0, 0],X_t[:, 0, 1]
    # y1_t, y2_t = X_t[:, 0, 0], X_t[:, 0, 1]
    # x1min = x1_t.min()
    # x1max = x1_t.max()
    # x2min = x2_t.min()
    # x2max = x2_t.max()
    # x1min = np.minimum(x1min,x2min)
    # x1max = np.maximum(x1max, x2max)
    # x2min = x1min
    # x2max = x2max
    # X1_t, X2_t =  np.mgrid[x1min:x2max:100j, x2min:x2max:100j]
    # positions = np.vstack([X1_t.ravel(), X2_t.ravel()])
    # values2d = np.vstack([x1_t, x2_t])
    # # kernel_x = stats.gaussian_kde(values2d)
    # Z = np.reshape(kernel_x(positions).T, X1_t.shape)
    # # axs[1].scatter(X_t[:, 0, 0], X_t[:, 0, 1], c=colors, s=10)
    # axs[0,1].imshow(np.rot90(Z), cmap=plt.cm.Reds,extent=[x1min, x1max, x2min, x2max])
    #
    # axs[0,1].set(xlim=(x1min, x1max), ylim=(x2min, x2max))
    # axs[0,1].set_aspect('equal', 'box')
    #
    # axs[1, 0].plot(np.arange(0,prms.N_p,1),Y_t[:, 0, 0],'ko',ms=0.25)
    # axs[1, 0].set(xlim=(0, prms.N_p), ylim=(np.min(Y_t[:, 0, 0]), np.max(Y_t[:, 0, 0])))
    # #
    # axs[1, 1].plot(np.arange(0,prms.N_p,1),Y_t[:, 0, 1],'ko',ms=0.25)
    # axs[1, 1].set(xlim=(0, prms.N_p), ylim=(np.min(Y_t[:, 0, 1]), np.max(Y_t[:, 0, 1])))

    # kappa1 = np.zeros(Vgrid.shape)
    # kappa2 = np.zeros(Vgrid.shape)
    # sigma=0.01
    # for ii in range(0,kappa1.shape[0]):
    #     for jj in range(0, kappa1.shape[1]):
    #         for k in range(0,prms.N_p):
    #             y1=Y_t[k, 0, 0]
    #             y2 = Y_t[k, 0, 1]
    #             kappa1[ii,jj]+=y1*np.exp(-( (X[ii,jj]-x1_t[k])**2+(Y[ii,jj]-x2_t[k])**2)/sigma)
    #             kappa2[ii, jj] += y2 * np.exp(-((X[ii, jj] - x1_t[k]) ** 2 + (Y[ii, jj] - x2_t[k]) ** 2) / sigma)
    #
    # pos1=axs[2, 0].contour(X, Y, kappa1, 20, cmap=cm.coolwarm)
    # axs[2,0].set(xlim=(x_min, x_max), ylim=(y_min, y_max))
    # fig.colorbar(pos1, ax=axs[2,0])
    # #
    # pos2 = axs[2, 1].contour(X, Y, kappa2, 20, cmap=cm.coolwarm)
    # axs[2, 1].set(xlim=(x_min, x_max), ylim=(y_min, y_max))
    # fig.colorbar(pos2, ax=axs[2, 1])

